//*****************************************************************************
// File Name	: main.c
// Title	: Proyecto de PDS
// Revision	: 0.1
// Notes	: version atmega8
// Target MCU	: Atmel AVR series - ATmega8
// Editor Tabs	: 2
//*****************************************************************************


//----- Include Files ---------------------------------------------------------
#include "extint.h"		// include external interrupt library
#include "timer.h"		// include timer function library (timing, PWM, etc)
#include "buffer.h"
#include <avr/io.h>		// include I/O definitions (port names, pin names, etc)
#include <avr/interrupt.h>	// include "signal" names (interrupt names)
#include <avr/sleep.h>
#include "global.h"		// include our global settings
#include "rprintf.h"
#include "extint.h"
#include "uart.h"
#include "a2d.h"		// include A/D converter function library


#define	int_enable		sbi(GICR, INT0)
#define	int_disable		cbi(GICR, INT0)

//******************************************************************************
//			DECLARACION DE FUNCIONES Y HANDLERS
//******************************************************************************

void timerHandler(void);  //Funcion que se invoca en las interrupciones por timer
void uartHandler(void);   //Funcion que se invoca en las interrupciones por uart
void timerRutine(void);   //Rutina para el caso de interrupcion por timer
void uartRutine(void);    //Rutina para el caso de interrupcion por uart



//******************************************************************************
//			MAIN Y VARIABLES GLOBALES
//*****************************************************************************

//Contadores globales
uint16_t iCounter = 0, iCounterExt = 0;
uint8_t  iRxmsg = 0;
uint8_t buff[3];
int index=0;

//Flag globales
uint8_t bTimerFlag = 0;
uint8_t bUartFlag = 0;

//variables de estado
int vpin = 0,state = -1;
char vnum = 'g';
uint16_t interval[8];


////////////////////////////////////////////////////////////
//                        MAIN
////////////////////////////////////////////////////////////

int main(void){

  u08 div = TIMER_CLK_DIV1024;
  
  
  //Inicializa timer
  timerInit();
  //Setea tiempo para la interrupcion
  timer0SetPrescaler(div);
  //Adjunta funcion local con la interrupcion de timer
  timerAttach(TIMER0OVERFLOW_INT, timerHandler);
  
  //Inicializa uart
  uartInit();
  //Adjunta funcion local con la interrupcion de recepcion
//   uartSetRxHandler(uartHandler);
  uartSetBaudRate(UART_DEFAULT_BAUD_RATE);
  
  //Inicializa rprintf con sendByte
  rprintfInit(uartSendByte);
  
  // turn on and initialize A/D converter
  a2dInit();
  // set the a2d prescaler (clock division ratio)
  // - a lower prescale setting will make the a2d converter go faster
  // - a higher setting will make it go slower but the measurements
  //   will be more accurate
  // - other allowed prescale values can be found in a2d.h
  a2dSetPrescaler(ADC_PRESCALE_DIV32);
  
  // set the a2d reference
  // - the reference is the voltage against which a2d measurements are made
  // - other allowed reference values can be found in a2d.h
  a2dSetReference(ADC_REFERENCE_AVCC);
  
  //inicializando variables
  int i = 0;
  for(i = 0;i<8;i++) interval[i] = -1;
  for(i=0;i<3;i++) buff[i] = 'j';
  
  //Inicializando puertos
  outb(DDRD, 0xFF);	//SE DEFINEN COMO SALIDA EL PUERTO D
  outb(PORTD, 0x00);
  
  outb(DDRB, 0xFF);	//SE DEFINEN COMO SALIDA EL PUERTO B
  outb(PORTB, 0x00);
  
  outb(DDRC, 0x00);	//SE DEFINEN COMO ENTRADA EL PUERTO C
  outb(PORTC, 0x00);
  
  sbi(PORTD, 2);
  rprintf("Iniciando Main\n");
  //RUTINA PRINCIPAL
  while(1){
  
    //Espera a que llegue un caracter
    while((uartReceiveByte(&(buff[iRxmsg])))&&(bTimerFlag != 1))
    //Compatibilidad con codigo anterior
    if(bTimerFlag == 0)
      uartHandler();
    
    //Rutina para el caso de interrupcion por timer
    if(bTimerFlag == 1){
      if(iCounter%15 == 0){
        iCounterExt++;
        timerRutine();
      }
      PORTD ^= 0x04;
      bTimerFlag = 0;
    }
    
    //Rutina para el caso de recepcion de datos serie
    if(bUartFlag == 1){
      uartRutine();
      bUartFlag = 0;
    }
    
    // save power by idling the processor
    set_sleep_mode(SLEEP_MODE_IDLE);
    sleep_mode();
    
  }

}


//******************************************************************************
//			IMPLEMENTACION DE FUNCIONES Y HANDLERS
//******************************************************************************

//Funcion que se invoca en las interrupciones por timer
void timerHandler(void){
  iCounter++;
  bTimerFlag = 1;
}

//Funcion que se invoca en las interrupciones por uart
void  uartHandler(void){
  bUartFlag = 1;
}

//Rutina para el caso de interrupcion por timer
void timerRutine(void){
  int i = 0;
  //rprintf("Timer Interrupt: %d\n",iCounterExt);
  for(i=0; i<8; i++){
    if((interval[i] != -1) && (iCounterExt%interval[i] == 0))
      //FIXME cambiar rprintf("%d",((i<<8)&0xff00|(a2dConvert8bit(i))));
      rprintf("Channel %d: %d   \r\n", i, a2dConvert8bit(i));
  }
}

//Rutina para el caso de recepcion de datos serie
void uartRutine(void){
  iRxmsg++;
  if(2 < iRxmsg)
    iRxmsg = 2;
  if(iRxmsg == 1){
    // FIXME if((buff[iRxmsg-1])&0x80)      buff[iRxmsg-1] = (buff[iRxmsg-1]>>4)&0x07;
    switch(buff[iRxmsg -1]){
      //valvulas
      case 'a':
      case 'b':
      case 'c':
      case 'd':
      case 'e':
        vnum = 'd';
        vpin = (int) (buff[iRxmsg -1] - 'a' + 3);
        break;
      //valvulas
      case 'f':
      case 'g':
      case 'h':
        vnum = 'b';
        vpin = (int) (buff[iRxmsg -1] - 'f');
        break;
      //sensores
      //FIXME cambiar valores por: 0x01, 0x02 0x03...0x8
      case 'x':
      case 'y':
      case 'z':
        index = (int) (buff[iRxmsg -1] - 'x');//FIXME eliminar -'x'
        break;
    }
  } else if(iRxmsg == 2){
    switch(buff[iRxmsg -2]){
      //valvulas
      case 'a':
      case 'b':
      case 'c':
      case 'd':
      case 'e':
      case 'f':
      case 'g':
      case 'h':
        state = (int) (buff[iRxmsg -1] - '0');
        break;
      //sensores
      case 'x':
      case 'y':
      case 'z':
        //FIXME cambiar mascara
        interval[index] = (uint16_t) ( ((buff[iRxmsg -2]<<8)&0x0000) | (buff[iRxmsg -1] - '0'));
        break;
    }
    iRxmsg = 0;
    
    //Activa o desactiva valvulas
    switch(vnum){
      case 'b':
        if(state == 0)
          cbi(PORTB,vpin);
        else if(state == 1)
          sbi(PORTB,vpin);
        break;
      case 'c':
        if(state == 0)
          cbi(PORTC,vpin);
        else if(state == 1)
          sbi(PORTC,vpin);
        break;
      case 'd':
        if(state == 0)
          cbi(PORTD,vpin);
        else if(state == 1)
          sbi(PORTD,vpin);
        break;
    }
    state = -1;
  }
}
